@@ -1768,6 +1768,14 @@ func TestFilterFlowerAddDel(t *testing.T) {
 	}
 
 	testMask := net.CIDRMask(24, 32)
+	srcMac, err := net.ParseMAC("2C:54:91:88:C9:E3")
+	if err != nil {
+		t.Fatal(err)
+	}
+	destMac, err := net.ParseMAC("2C:54:91:88:C9:E5")
+	if err != nil {
+		t.Fatal(err)
+	}
 
 	ipproto := new(nl.IPProto)
 	*ipproto = nl.IPPROTO_TCP
@@ -1790,10 +1798,19 @@ func TestFilterFlowerAddDel(t *testing.T) {
 		EncSrcIPMask:  testMask,
 		EncDestPort:   8472,
 		EncKeyId:      1234,
+		SrcMac:        srcMac,
+		DestMac:       destMac,
 		IPProto:       ipproto,
 		DestPort:      1111,
 		SrcPort:       1111,
 		Actions: []Action{
+			&VlanAction{
+				ActionAttrs: ActionAttrs{
+					Action: TC_ACT_PIPE,
+				},
+				Action: TCA_VLAN_ACT_PUSH,
+				VlanID: 1234,
+			},
 			&MirredAction{
 				ActionAttrs: ActionAttrs{
 					Action: TC_ACT_STOLEN,
@@ -1871,8 +1888,31 @@ func TestFilterFlowerAddDel(t *testing.T) {
 	if filter.SrcPort != flower.SrcPort {
 		t.Fatalf("Flower SrcPort doesn't match")
 	}
+	if !(filter.SrcMac.String() == flower.SrcMac.String()) {
+		t.Fatalf("Flower SrcMac doesn't match")
+	}
+	if !(filter.DestMac.String() == flower.DestMac.String()) {
+		t.Fatalf("Flower DestMac doesn't match")
+	}
 
-	mia, ok := flower.Actions[0].(*MirredAction)
+	vla, ok := flower.Actions[0].(*VlanAction)
+	if !ok {
+		t.Fatal("Unable to find vlan action")
+	}
+
+	if vla.Attrs().Action != TC_ACT_PIPE {
+		t.Fatal("Vlan action isn't TC_ACT_PIPE")
+	}
+
+	if vla.Action != TCA_VLAN_ACT_PUSH {
+		t.Fatal("Second Vlan action isn't push")
+	}
+
+	if vla.VlanID != 1234 {
+		t.Fatal("Second Vlan action vlanId isn't correct")
+	}
+
+	mia, ok := flower.Actions[1].(*MirredAction)
 	if !ok {
 		t.Fatal("Unable to find mirred action")
 	}
@@ -1889,7 +1929,7 @@ func TestFilterFlowerAddDel(t *testing.T) {
 		t.Fatal("Incorrect mirred action stats")
 	}
 
-	ga, ok := flower.Actions[1].(*GenericAction)
+	ga, ok := flower.Actions[2].(*GenericAction)
 	if !ok {
 		t.Fatal("Unable to find generic action")
 	}
@@ -1917,6 +1957,146 @@ func TestFilterFlowerAddDel(t *testing.T) {
 		t.Fatal("Failed to remove filter")
 	}
 
+	filter = &Flower{
+		FilterAttrs: FilterAttrs{
+			LinkIndex: link.Attrs().Index,
+			Parent:    MakeHandle(0xffff, 0),
+			Priority:  1,
+			Protocol:  unix.ETH_P_8021Q,
+		},
+		EthType: unix.ETH_P_8021Q,
+		VlanId:  2046,
+		Actions: []Action{
+			&VlanAction{
+				ActionAttrs: ActionAttrs{
+					Action: TC_ACT_PIPE,
+				},
+				Action: TCA_VLAN_ACT_POP,
+			},
+			&MirredAction{
+				ActionAttrs: ActionAttrs{
+					Action: TC_ACT_STOLEN,
+				},
+				MirredAction: TCA_EGRESS_REDIR,
+				Ifindex:      redir.Attrs().Index,
+			},
+		},
+	}
+
+	if err := FilterAdd(filter); err != nil {
+		t.Fatal(err)
+	}
+
+	time.Sleep(time.Second)
+	filters, err = FilterList(link, MakeHandle(0xffff, 0))
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(filters) != 1 {
+		t.Fatal("Failed to add filter")
+	}
+	flower, ok = filters[0].(*Flower)
+	if !ok {
+		t.Fatal("Filter is the wrong type")
+	}
+
+	if filter.VlanId != flower.VlanId {
+		t.Fatalf("Flower VlanId doesn't match")
+	}
+
+	vla, ok = flower.Actions[0].(*VlanAction)
+	if !ok {
+		t.Fatal("Unable to find vlan action")
+	}
+
+	if vla.Attrs().Action != TC_ACT_PIPE {
+		t.Fatal("Vlan action isn't TC_ACT_PIPE")
+	}
+
+	if vla.Action != TCA_VLAN_ACT_POP {
+		t.Fatal("First Vlan action isn't pop")
+	}
+
+	mia, ok = flower.Actions[1].(*MirredAction)
+	if !ok {
+		t.Fatal("Unable to find mirred action")
+	}
+
+	if mia.Attrs().Action != TC_ACT_STOLEN {
+		t.Fatal("Mirred action isn't TC_ACT_STOLEN")
+	}
+
+	if mia.Timestamp == nil || mia.Timestamp.Installed == 0 {
+		t.Fatal("Incorrect mirred action timestamp")
+	}
+
+	if mia.Statistics == nil {
+		t.Fatal("Incorrect mirred action stats")
+	}
+
+	if err := FilterDel(filter); err != nil {
+		t.Fatal(err)
+	}
+	filters, err = FilterList(link, MakeHandle(0xffff, 0))
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(filters) != 0 {
+		t.Fatal("Failed to remove filter")
+	}
+
+	classId := MakeHandle(1, 101)
+
+	filter = &Flower{
+		FilterAttrs: FilterAttrs{
+			LinkIndex: link.Attrs().Index,
+			Parent:    MakeHandle(0xffff, 0),
+			Priority:  1,
+			Protocol:  unix.ETH_P_ALL,
+		},
+
+		EthType:         unix.ETH_P_IP,
+		IPProto:         ipproto,
+		ClassId:         classId,
+		SrcPortRangeMin: 1000,
+		SrcPortRangeMax: 2000,
+	}
+	if err := FilterAdd(filter); err != nil {
+		t.Fatal(err)
+	}
+
+	time.Sleep(time.Second)
+	filters, err = FilterList(link, MakeHandle(0xffff, 0))
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(filters) != 1 {
+		t.Fatal("Failed to add filter")
+	}
+	flower, ok = filters[0].(*Flower)
+	if !ok {
+		t.Fatal("Filter is the wrong type")
+	}
+	if filter.ClassId != flower.ClassId {
+		t.Fatalf("Flower ClassId doesn't match")
+	}
+	if filter.SrcPortRangeMin != flower.SrcPortRangeMin {
+		t.Fatalf("Flower SrcPortRangeMin doesn't match")
+	}
+	if filter.SrcPortRangeMax != flower.SrcPortRangeMax {
+		t.Fatalf("Flower SrcPortRangeMax doesn't match")
+	}
+	if err := FilterDel(filter); err != nil {
+		t.Fatal(err)
+	}
+	filters, err = FilterList(link, MakeHandle(0xffff, 0))
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(filters) != 0 {
+		t.Fatal("Failed to remove filter")
+	}
+
 	if err := QdiscDel(qdisc); err != nil {
 		t.Fatal(err)
 	}
@@ -2471,3 +2651,135 @@ func TestFilterChainAddDel(t *testing.T) {
 		t.Fatal("Failed to remove qdisc")
 	}
 }
+
+func TestFilterSampleAddDel(t *testing.T) {
+	minKernelRequired(t, 4, 11)
+	if _, err := GenlFamilyGet("psample"); err != nil {
+		t.Skip("psample genetlink family unavailable - is CONFIG_PSAMPLE enabled?")
+	}
+
+	tearDown := setUpNetlinkTest(t)
+	defer tearDown()
+	if err := LinkAdd(&Ifb{LinkAttrs{Name: "foo"}}); err != nil {
+		t.Fatal(err)
+	}
+	link, err := LinkByName("foo")
+	if err != nil {
+		t.Fatal(err)
+	}
+	if err := LinkSetUp(link); err != nil {
+		t.Fatal(err)
+	}
+
+	qdisc := &Ingress{
+		QdiscAttrs: QdiscAttrs{
+			LinkIndex: link.Attrs().Index,
+			Handle:    MakeHandle(0xffff, 0),
+			Parent:    HANDLE_INGRESS,
+		},
+	}
+	if err := QdiscAdd(qdisc); err != nil {
+		t.Fatal(err)
+	}
+	qdiscs, err := SafeQdiscList(link)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	found := false
+	for _, v := range qdiscs {
+		if _, ok := v.(*Ingress); ok {
+			found = true
+			break
+		}
+	}
+	if !found {
+		t.Fatal("Qdisc is the wrong type")
+	}
+
+	sample := NewSampleAction()
+	sample.Group = 7
+	sample.Rate = 12
+	sample.TruncSize = 200
+
+	classId := MakeHandle(1, 1)
+	filter := &MatchAll{
+		FilterAttrs: FilterAttrs{
+			LinkIndex: link.Attrs().Index,
+			Parent:    MakeHandle(0xffff, 0),
+			Priority:  1,
+			Protocol:  unix.ETH_P_ALL,
+		},
+		ClassId: classId,
+		Actions: []Action{
+			sample,
+		},
+	}
+
+	if err := FilterAdd(filter); err != nil {
+		t.Fatal(err)
+	}
+
+	filters, err := FilterList(link, MakeHandle(0xffff, 0))
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(filters) != 1 {
+		t.Fatal("Failed to add filter")
+	}
+	mf, ok := filters[0].(*MatchAll)
+	if !ok {
+		t.Fatal("Filter is the wrong type")
+	}
+
+	if len(mf.Actions) < 1 {
+		t.Fatalf("Too few Actions in filter")
+	}
+	if mf.ClassId != classId {
+		t.Fatalf("ClassId of the filter is the wrong value")
+	}
+
+	lsample, ok := mf.Actions[0].(*SampleAction)
+	if !ok {
+		t.Fatal("Unable to find sample action")
+	}
+	if lsample.Group != sample.Group {
+		t.Fatalf("Inconsistent sample action group")
+	}
+	if lsample.Rate != sample.Rate {
+		t.Fatalf("Inconsistent sample action rate")
+	}
+	if lsample.TruncSize != sample.TruncSize {
+		t.Fatalf("Inconsistent sample truncation size")
+	}
+
+	if err := FilterDel(filter); err != nil {
+		t.Fatal(err)
+	}
+	filters, err = FilterList(link, MakeHandle(0xffff, 0))
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(filters) != 0 {
+		t.Fatal("Failed to remove filter")
+	}
+
+	if err := QdiscDel(qdisc); err != nil {
+		t.Fatal(err)
+	}
+	qdiscs, err = SafeQdiscList(link)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	found = false
+	for _, v := range qdiscs {
+		if _, ok := v.(*Ingress); ok {
+			found = true
+			break
+		}
+	}
+	if found {
+		t.Fatal("Failed to remove qdisc")
+	}
+}
